#include "encompass.h"
#include <libgnomevfs/gnome-vfs-mime.h>

static gchar * encompass_load_image (gchar * imgurl) {
  const gchar * header = g_strdup ("<html><head><title>Image: ");
  const gchar * body = g_strdup ("</title></head><body><img src=\"");
  const gchar * footer = g_strdup ("\" border=\"0\"></body></html>");

  return g_strdup_printf ("%s%s%s%s%s", header, g_basename (imgurl),
			  body, imgurl, footer);
}

gchar * encompass_parse_url (gchar * url, EncompassURI * base) {
  EncompassURI * tmpurl;
  gchar * blah;

  tmpurl = encompass_uri_dup (base, ENCOMPASS_URI_DUP_ALL);

  if (!strncmp (url, "//", strlen ("//"))) {
    blah = g_strconcat ("http:", url, NULL);
    if (tmpurl) {
      encompass_uri_destroy (tmpurl);
      tmpurl = encompass_uri_new (blah);
    }
  } else if (!strncmp (url, "/", strlen ("/"))) {
    blah = g_strdup (url);
    if (tmpurl) {
      encompass_uri_set_path (tmpurl, g_strdup (blah));
    }
  } else if (!strncmp (url, "../", strlen ("../"))) {
    gchar * tmpblah;

    tmpblah = g_strndup (g_dirname (base->path),
			 strlen (g_dirname (base->path)) -
			 strlen (strrchr (g_dirname (base->path), '/')));
    blah = g_strdup_printf ("%s/%s", g_strdup (tmpblah),
			    g_strdup (url + strlen ("../")));
    if (tmpurl) {
      encompass_uri_set_path (tmpurl, g_strdup (blah));
    }
  } else if (!strncmp (url, "./", strlen ("./"))) {
    blah = g_strdup_printf ("%s/%s", g_dirname (base->path),
			    g_strdup (url + strlen ("./")));
    if (tmpurl) {
      encompass_uri_set_path (tmpurl, g_strdup (blah));
    }
  } else if (strchr (url, ':')) {
    tmpurl = encompass_uri_new (g_strdup (url));
  } else if (!g_strcasecmp (g_dirname (base->path), "/")) {
    blah = g_strdup_printf ("/%s", g_strdup (url));
    if (tmpurl) {
      encompass_uri_set_path (tmpurl, g_strdup (blah));
    }
  } else if (!strncmp (url, "#", strlen ("#"))) {
    encompass_uri_set_reference (tmpurl, g_strdup (url) + 1);
  } else {
    blah = g_strdup_printf ("%s/%s", g_dirname (base->path), g_strdup (url));
    if (tmpurl) {
      encompass_uri_set_path (tmpurl, g_strdup (blah));
    }
  }
  if (tmpurl) {
    return g_strdup (encompass_uri_to_string
		     (encompass_uri_dup (tmpurl, ENCOMPASS_URI_DUP_ALL)));
  } else {
    return url;
  }
}

gchar * encompass_validate_uri (gchar * url) {
  gchar * tmpurl = NULL;
  gchar * testlocal;

  if (url && *url) {
    if (strchr (url, ':')) {
      tmpurl = g_strdup (url);
    } else {
      testlocal = g_strconcat (g_get_current_dir (), "/",
			       g_strdup (url), NULL);
      if (!strncmp (url, "ftp.", strlen ("ftp."))) {
	tmpurl = g_strdup_printf ("ftp://%s", g_strdup (url));
      } else if (!strncmp (url, "/", strlen ("/"))) {
	tmpurl = g_strdup_printf ("file://%s", g_strdup (url));
      } else if (g_file_exists (g_strdup (testlocal))) {
	tmpurl = g_strdup_printf ("file://%s", g_strdup (testlocal));
      } else {
	tmpurl = g_strdup_printf ("http://%s", g_strdup (url));
      }
      g_free (testlocal);
    }
  }
  if (strstr (tmpurl, ":/")) {
    if (!strrchr ((gchar *) strstr (tmpurl, ":/") + 3, '/') &&
	tmpurl[strlen (tmpurl)] != '/') {
      tmpurl = g_strdup_printf ("%s/", tmpurl);
    }
  }
  return tmpurl;
}

static void encompass_go_list_clear (gpointer data, gpointer udata) {
  HistoryItem * item = (HistoryItem *) data;
	
  if(item->widget) {
    gtk_widget_destroy(item->widget);
  }
  item->widget = NULL;
}

void encompass_goto_url (BrowserWindow * window, gchar * url,
			 int history) {
  EncompassURI * uri;
  HistoryItem * item;
  int tmp, i;
  gchar * real_url;

  real_url = encompass_validate_uri (url);
  gtk_widget_set_sensitive (window->stop_tool_item, TRUE);
  if (real_url && *real_url) {
    uri = encompass_uri_new (g_strdup (real_url));

    gtk_widget_set_sensitive (window->stop_tool_item, TRUE);
    if (!strncmp (gnome_vfs_mime_type_from_name (real_url),
		  g_strdup ("image/"), strlen (g_strdup ("image/")))) {
      gchar * imgpage;
      gchar * newurl = g_strdup (encompass_parse_url (g_strdup (real_url),
						      window->base));

      window->handle = gtk_html_begin (GTK_HTML (window->html));

      imgpage = g_strdup (encompass_load_image (g_strdup (newurl)));
      encompass_set_base (GTK_HTML (window->html), g_strdup (newurl),
			  window);
      gtk_html_write (GTK_HTML (window->html), window->handle,
		      g_strdup (imgpage), strlen (imgpage));
      gtk_html_end (GTK_HTML (window->html), window->handle,
		    GTK_HTML_STREAM_OK);
      g_free (newurl);
      g_free (imgpage);
    } else if (!g_strcasecmp (uri->protocol, "file")) {
      window->handle = gtk_html_begin (GTK_HTML (window->html));
      encompass_set_base (GTK_HTML (window->html), real_url, window);
      encompass_load_file (window, uri, window->handle);
    } else if (!g_strcasecmp (uri->protocol, "mailto")) {
      gtk_widget_set_sensitive (window->stop_tool_item, FALSE);
      gnome_url_show (encompass_uri_to_string (uri));
      encompass_uri_destroy (uri);
      return;
    } else if (!g_strcasecmp (uri->protocol, "http") ||
	       !g_strcasecmp (uri->protocol, "https")) {
      window->handle = gtk_html_begin (GTK_HTML (window->html));
      encompass_set_base (GTK_HTML (window->html), real_url, window);
      encompass_http_load_url (window, uri, window->handle, FALSE, "");
    }
    if (uri->reference && *(uri->reference)) {
      gtk_html_jump_to_anchor (GTK_HTML (window->html), uri->reference);
    }
    encompass_uri_destroy (uri);
    if (!history) {
      if(window->go_position) {
	tmp = window->go_position;
	while(tmp) {
	  item = g_list_nth_data(window->go_list, --tmp);
	  if (item) {
	    window->go_list = g_list_remove(window->go_list, item);
	    if(item->url)
	      g_free(item->url);
	    if(item->title)
	      g_free(item->title);
	    if(item->url)
	      gtk_widget_destroy(item->widget);
	    g_free(item);
	  }
	}
	window->go_position = 0;
      }
      tmp = g_list_length(window->go_list);
      while(tmp > MAX_GO_ENTRIES) {
	item = g_list_nth_data(window->go_list, MAX_GO_ENTRIES);
	
	if(item->url)
	  g_free(item->url);
	if(item->title)
	  g_free(item->title);
	if(item->url)
	  gtk_widget_destroy(item->widget);
	g_free(item);

	window->go_list = g_list_remove(window->go_list, item);
	tmp--;
      }
      item = NULL;
      item = g_new0 (HistoryItem, 1);
    
      item->url = g_strdup (real_url);
      item->title = g_strdup (gtk_html_get_title (GTK_HTML (window->html)));

      g_list_foreach (window->go_list, encompass_go_list_clear, NULL);
      window->go_list = g_list_prepend (window->go_list, item);

      tmp = g_list_length (window->go_list);
      window->group = NULL;

      for (i = 0; i < tmp; i++) {
	item = g_list_nth_data (window->go_list, i);
	if (item->title) {
	  if (strlen (item->title) > 32) {
	    gchar * tooltiped;

	    tooltiped = g_strconcat (g_strndup (item->title, 29), "...", NULL);
	    item->widget = gtk_radio_menu_item_new_with_label (window->group,
							       tooltiped);
	    gtk_tooltips_set_tip (window->tips, item->widget, item->title, NULL);
	    g_free (tooltiped);
	  } else {
	    item->widget = gtk_radio_menu_item_new_with_label (window->group,
							       item->title);
	  }
	} else {
	  item->widget = gtk_radio_menu_item_new_with_label (window->group,
							     item->url);
	}
	gtk_signal_connect (GTK_OBJECT (item->widget), "toggled",
			    GTK_SIGNAL_FUNC (encompass_history_go_list),
			    window);
	window->group =
	  gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item->widget));
	if (i == 0) {
	  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item->widget),
					  TRUE);
	  window->go_position = 0;
	}
	encompass_go_menu_append (window, item->widget);
	gtk_widget_show (item->widget);
      }
      window->group = g_slist_reverse (window->group);
    }  else {
    item = g_list_nth_data (window->go_list, window->go_position);
    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item->widget), TRUE);
    }
  }
  tmp = g_list_length (window->go_list);
  if (window->go_position == 0 || tmp < 2) {
    encompass_set_fore_buttons (window, FALSE);
  } else {
    encompass_set_fore_buttons (window, TRUE);
  }
  if (window->go_position == (tmp - 1) || tmp < 2) {
    encompass_set_back_buttons (window, FALSE);
  } else {
    encompass_set_back_buttons (window, TRUE);
  }
  for (tmp = 0; tmp < g_list_length (window->go_list); tmp++) {
    item = g_list_nth_data (window->go_list, tmp);
    if (item) {
      GTK_CHECK_MENU_ITEM (item->widget)->active = FALSE;
    }
  }
  item = g_list_nth_data (window->go_list, window->go_position);
  if (item) {
    GTK_CHECK_MENU_ITEM (item->widget)->active = TRUE;
  }
  gtk_widget_set_sensitive (window->stop_tool_item, FALSE);
  gnome_appbar_set_progress (GNOME_APPBAR (window->status), 0.0);
  gnome_appbar_set_status (GNOME_APPBAR (window->status), "");
  gnome_appbar_refresh (GNOME_APPBAR (window->status));
}

void encompass_entry_goto_url (GtkWidget * widget, BrowserWindow * window) {
  gchar * url;

  url = g_strdup (gtk_entry_get_text (GTK_ENTRY (window->entry)));
  encompass_goto_url (window, g_strdup (url), 0);
  g_free (url);
}
