#include "encompass.h"
#include <gtkhtml/htmlengine.h>
#include <gtkhtml/htmlobject.h>
#include <gtkhtml/htmlimage.h>
#include <gtkhtml/htmllinktext.h>
#include <gtkhtml/htmlselection.h>
#include <gtkhtml/htmlselect.h>

void encompass_url_hover (GtkHTML * html, const gchar * url,
			  BrowserWindow * window) {
  if (url) {
    gnome_appbar_set_status (GNOME_APPBAR (window->status), url);
  } else {
    gnome_appbar_set_status (GNOME_APPBAR (window->status), "");
  }
}

void encompass_title_changed (GtkHTML * html, const gchar * title,
			      BrowserWindow * window) {
  gchar * s;
  s = g_strconcat ("Encompass: ", title, NULL);
  gtk_window_set_title (GTK_WINDOW (window->window), g_strdup (s));
  g_free (s);
}

void encompass_load_home_page (GtkWidget * widget, BrowserWindow * window) {
  gchar * blah;

  blah = g_strconcat (g_get_home_dir (), "/public_html/index.html", NULL);
  if (g_file_exists (blah)) {
    encompass_goto_url (window, encompass_parse_url (blah, window->base), 0);
  }
  g_free (blah);
}

void encompass_url_requested (GtkHTML * html, const gchar * url,
			      GtkHTMLStream * handle, BrowserWindow * window) {
  EncompassURI * uri;

  uri = encompass_uri_new (encompass_parse_url ((gchar *)url, window->base));
  if (!strcasecmp (uri->protocol, "file")) {
    encompass_load_file (window, uri, handle);
  } else if (!g_strcasecmp (uri->protocol, "http") ||
	     !g_strcasecmp (uri->protocol, "https")) {
    encompass_http_load_url (window, uri, handle, FALSE, "");
  } else {
    printf ("%s\n", url);
  }
}

void encompass_submit_form (GtkHTML *html, const gchar *method,
			    const gchar *action, const gchar *encoding,
			    BrowserWindow * window) {
  if (!g_strcasecmp (method, "GET")) {
    gchar * blah;
    blah = g_strconcat (action, "?", encoding, NULL);
    encompass_goto_url (window, encompass_parse_url (blah, window->base), 0);
    g_free (blah);
  } else if (!g_strcasecmp (method, "POST")) {
    gchar ** pieces = g_strsplit (encoding, "&", -1);
    gchar * realurl;

    if (!action) {
      gchar * tmpurl = encompass_uri_to_string (window->base);
      printf ("DEBUG: New URL: %s\n", tmpurl);
      realurl = encompass_parse_url (tmpurl, window->base);
      g_free (tmpurl);
    } else {
      realurl = encompass_parse_url ((gchar *)action, window->base);
    }
      
    if (pieces) {
      gchar * post_body = g_strjoinv ("\r\n", pieces);
      EncompassURI * uri = encompass_uri_new (realurl);

      window->handle = gtk_html_begin (html);
      encompass_http_load_url (window, uri, window->handle,
			       TRUE, post_body);
    }
    g_free (realurl);
  } else {
    g_warning ("Encompass: Unsupported submit method '%s'\n", method);
  }
}

void encompass_button_pressed (GtkWidget * widget, GdkEventButton * event,
			       BrowserWindow * window) {

  if (event->type == GDK_BUTTON_PRESS) {
    if (event->button == 2) {
      HTMLObject * object;
      guint offset;

      object = html_engine_get_object_at (GTK_HTML (window->html)->engine,
					  event->x + GTK_HTML (window->html)->engine->x_offset,
					  event->y + GTK_HTML (window->html)->engine->y_offset,
					  &offset, FALSE);

      if (object != NULL && ((HTML_OBJECT_TYPE (object) == HTML_TYPE_LINKTEXT)
	  || (HTML_OBJECT_TYPE (object) == HTML_TYPE_IMAGE
	      && html_object_get_url (object)))) {
	gchar * url = (gchar *) html_object_get_url (object);

	encompass_create_window (NULL,
				 encompass_parse_url (url, window->base));
      }
    } else if (event->button == 3) {
      encompass_menu_popup (event, window);
    }
  }
}

void encompass_set_base (GtkHTML * html, const gchar * url,
			 BrowserWindow * window) {
  gtk_entry_set_text (GTK_ENTRY (window->entry), url);
  if (!window->base) {
    window->base = encompass_uri_new (url);
  } else {
    encompass_uri_destroy (window->base);
    window->base = encompass_uri_new (url);
  }
}

void encompass_link_clicked (GtkHTML * html, const gchar * url,
			     BrowserWindow * window) {
  g_return_if_fail (html != NULL);

  if (*url == '#') {
    gtk_html_jump_to_anchor (html, url + 1);
  } else {
    encompass_goto_url (window, encompass_parse_url ((gchar *)url,
						     window->base), 0);
  }
}

void encompass_refresh (GtkWidget * widget, BrowserWindow * window) {
  gtk_widget_draw (window->html, NULL);
}

void encompass_load_done (GtkWidget * widget, BrowserWindow * window) {
  gtk_widget_grab_focus (window->html);
}
