#include "encompass.h"

gboolean event_loop = FALSE;
gint counter = 0;

void encompass_http_stop (GtkWidget * widget, BrowserWindow * window) {
  int i;
  ne_session * session;

  /* test with penguinppc.org/~eb/penguin.html */
  for (i = 0; i < g_list_length (window->sessions); i++) {
    session = g_list_nth_data (window->sessions, i);
    window->sessions = g_list_remove (window->sessions, session);
    /*
      Why does this cause one or multiple segfaults?
    ne_close_connection (session);
    */
  }
}

static void encompass_http_redirect (WebThreadStuff * stuff,
				     const gchar * src,
				     const gchar * dest) {
}

static int encompass_http_redirect_confirm (WebThreadStuff * stuff,
					    const gchar * src,
					    const gchar * dest) {
  return TRUE;
}

static void encompass_http_status (WebThreadStuff * stuff,
				   ne_conn_status status, const gchar * info) {
  GnomeAppBar * blah;
  blah = GNOME_APPBAR (GNOME_APP (stuff->window->window)->statusbar);

  g_mutex_lock (stuff->window->gtk_mtx);
  gdk_threads_enter ();

  switch (status) {
  case ne_conn_namelookup: {
    gchar * msg = g_strdup_printf (_("Looking up %s..."), info);
    gnome_appbar_set_status (blah, msg);
    g_free (msg);
    break;
  }
  case ne_conn_connecting: {
    gchar * msg = g_strdup_printf (_("Connecting to %s..."), info);
    gnome_appbar_set_status (blah, msg);
    g_free (msg);
    break;
  }
  case ne_conn_connected: {
    gchar * msg = g_strdup_printf (_("Connected to %s..."), info);
    gnome_appbar_set_status (blah, msg);
    g_free (msg);
    break;
  }
  default: {
    gnome_appbar_set_status (blah, "");
    break;
  }
  }
  gdk_threads_leave ();
  g_mutex_unlock (stuff->window->gtk_mtx);
}

static void encompass_http_progress (WebThreadStuff * stuff,
				     off_t progress, off_t total) {
  GnomeAppBar * blah;
  gfloat percent = (progress / total);

  blah = GNOME_APPBAR (GNOME_APP (stuff->window->window)->statusbar);
  
  g_mutex_lock (stuff->window->gtk_mtx);
  gdk_threads_enter ();

  if (percent > 1.0) {
    percent = 1.0;
  }
  if (percent < 0.0) {
    percent = 0.0;
  }
  gnome_appbar_set_progress (blah, percent);

  gdk_threads_leave ();
  g_mutex_unlock (stuff->window->gtk_mtx);
}

static void encompass_http_read_block (WebThreadStuff * stuff,
				       const gchar * buf,
				       size_t len) {
  g_mutex_lock (stuff->window->gtk_mtx);
  gdk_threads_enter ();

  if (len) {
    gtk_html_write (GTK_HTML (stuff->window->html), stuff->handle, buf, len);
  }
  gdk_threads_leave ();
  g_mutex_unlock (stuff->window->gtk_mtx);
}

static int encompass_http_auth (WebThreadStuff * stuff,
				  const gchar * realm, int attempt,
				  gchar * username, gchar * password) {
  gint retval = 1;

  g_mutex_lock (stuff->window->gtk_mtx);
  gdk_threads_enter ();
  {
    GtkWidget * authbox = NULL;
    GtkWidget * userbox = NULL, * passbox = NULL;

    authbox = gnome_message_box_new (_("Please type in your user name "
				       "and password."),
				     GNOME_MESSAGE_BOX_QUESTION,
				     GNOME_STOCK_BUTTON_OK,
				     GNOME_STOCK_BUTTON_CANCEL, NULL);
    userbox = gtk_entry_new ();
    gtk_container_add (GTK_CONTAINER (GNOME_DIALOG (authbox)->vbox), userbox);
    gtk_widget_show (userbox);

    passbox = gtk_entry_new ();
    gtk_container_add (GTK_CONTAINER (GNOME_DIALOG (authbox)->vbox), passbox);
    gtk_widget_show (passbox);
    gtk_widget_show (userbox);

    retval = gnome_dialog_run_and_close (GNOME_DIALOG (authbox));
    switch (retval) {
    case 0: {
      username = g_strdup (gtk_entry_get_text (GTK_ENTRY (userbox)));
      password = g_strdup (gtk_entry_get_text (GTK_ENTRY (passbox)));
      break;
    }
    default:
      break;
    }
  }
  gdk_threads_leave ();
  g_mutex_unlock (stuff->window->gtk_mtx);

  printf ("DEBUG: AUTH: %d\n", retval);
  return retval;
}

static void encompass_http_load (WebThreadStuff * stuff) {
  BrowserWindow * window = stuff->window;
  EncompassURI * uri = encompass_uri_dup (stuff->uri, ENCOMPASS_URI_DUP_ALL);
  gchar * referer = encompass_uri_to_string (window->base);
  ne_session * session;
  ne_request * request;

  g_mutex_unlock (window->data_mtx);

  session = ne_session_create (uri->protocol, uri->hostname, uri->port);
  window->sessions = g_list_append (window->sessions, session);

  ne_set_read_timeout (session, 4);
  ne_set_server_auth (session, (void *) encompass_http_auth, stuff);
  ne_set_proxy_auth (session, (void *) encompass_http_auth, stuff);
  ne_set_status (session, (void *) encompass_http_status, stuff);
  ne_set_progress (session, (void *) encompass_http_progress, stuff);
  if (g_getenv ("http_proxy")) {
    EncompassURI * proxy = encompass_uri_new (g_getenv ("http_proxy"));

    ne_session_proxy (session, proxy->hostname, proxy->port);
    encompass_uri_destroy (proxy);
  }
  ne_set_useragent (session, "Mozilla/5 compatible; Encompass " VERSION);
  ne_redirect_register (session,
			(void *) encompass_http_redirect_confirm,
			(void *) encompass_http_redirect, stuff);
  if (stuff->isposting) {
    printf ("DEBUG: We are posting\n");

    g_mutex_lock (window->gtk_mtx);
    gdk_threads_enter ();

    encompass_set_base (GTK_HTML (window->html),
			(const gchar *)encompass_uri_to_string (uri), window);

    gdk_threads_leave ();
    g_mutex_unlock (window->gtk_mtx);

    request = ne_request_create (session, "POST", uri->path);
    ne_add_request_header(request, "Content-Type",
			  "application/x-www-form-urlencoded");
    
    ne_set_request_body_buffer (request, g_strdup (stuff->post_body),
				strlen (stuff->post_body));
    printf ("DEBUG: Body:\n %s\n", stuff->post_body);
    stuff->isposting = FALSE;
  } else {
    request = ne_request_create (session, "GET", uri->path);
  }
  ne_add_response_header_handler (request, "Set-Cookie",
				  (void *) encompass_get_cookies, stuff);
  ne_add_request_header(request, "Referer", g_strdup (referer));
  ne_add_response_body_reader (request, ne_accept_always,
			       (void *) encompass_http_read_block, stuff);

  switch (ne_request_dispatch (request)) {
  case NE_OK:
    break;
  default: {
    printf ("DEBUG: %s\n", ne_get_error (session));
    break;
  }
  }

  ne_request_destroy (request);
  window->sessions = g_list_remove (window->sessions, session);
  ne_session_destroy (session);

  if (stuff->post_body) {
    g_free (stuff->post_body);
    stuff->post_body = NULL;
  }

  g_mutex_lock (window->gtk_mtx);
  gdk_threads_enter ();

  gtk_html_end (GTK_HTML (window->html), stuff->handle,
		GTK_HTML_STREAM_OK);

  gdk_threads_leave ();
  g_mutex_unlock (window->gtk_mtx);
  g_mutex_lock (window->counter_mtx);
  counter--;
  g_mutex_unlock (window->counter_mtx);
}

void encompass_http_load_url (BrowserWindow * window, EncompassURI * uri,
			      GtkHTMLStream * handle, gboolean isposting,
			      gchar * post_body) {
  WebThreadStuff * stuff;
  pthread_t load_thread;
  gboolean start_loop;

  if (uri->port == 0) {
    if (!g_strcasecmp (uri->protocol, "https")) {
      uri->port = 443;
    } else {
      uri->port = 80;
    }
  }

  stuff = g_new0 (WebThreadStuff, 1);
  stuff->uri = uri;
  stuff->window = window;
  stuff->handle = handle;
  stuff->isposting = isposting;
  stuff->post_body = g_strdup (post_body);

  g_mutex_lock (window->counter_mtx);
  counter++;
  g_mutex_unlock (window->counter_mtx);

  g_mutex_lock(window->event_loop_mtx);
  start_loop = !event_loop;
  if (start_loop) {
    event_loop = TRUE;
  }
  g_mutex_unlock(window->event_loop_mtx);

  g_mutex_lock (window->data_mtx);
  pthread_create(&load_thread, NULL, (void *) encompass_http_load, stuff);
  g_mutex_lock (window->data_mtx);
  g_mutex_unlock (window->data_mtx);

  if (start_loop) {
    g_mutex_lock(window->counter_mtx);
    while (counter > 0) {
      g_mutex_unlock(window->counter_mtx);
      g_mutex_lock (window->gtk_mtx);
      gdk_threads_enter ();
      while(gtk_events_pending()) {
        gtk_main_iteration();
      }
      gdk_threads_leave ();
      g_mutex_unlock (window->gtk_mtx);
      sleep(0);
      g_mutex_lock(window->counter_mtx);
    }
    g_mutex_unlock(window->counter_mtx);
    g_mutex_lock(window->event_loop_mtx);
    event_loop = FALSE;
    g_mutex_unlock(window->event_loop_mtx);
  }
}

