#include "encompass.h"

void encompass_create_toolbars (BrowserWindow * window) {
  GtkWidget * label;
  GtkWidget * hbox;
  GtkWidget * urlpixmap;
  GtkWidget * toolbar;
  gchar * temp, * theme;

  /* Load the theme path information */
  theme = g_strconcat ("encompass/", window->prefs->themename, NULL);
  if (!g_file_exists (elysium_pixmap_file (theme)) ||
      g_file_test (elysium_pixmap_file (theme), G_FILE_TEST_ISDIR)) {
    window->prefs->themename = g_strdup ("default");
    if (theme) {
      g_free (theme);
    }
    theme = g_strconcat ("encompass/", window->prefs->themename, NULL);
  }

  /* Create the Navigation toolbar */
  window->navbar = gnome_dock_item_new ("navbar", GNOME_DOCK_ITEM_BEH_EXCLUSIVE);
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window->navbar), hbox);
  gtk_container_border_width (GTK_CONTAINER (window->navbar), 2);

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_box_pack_start (GTK_BOX (hbox), toolbar, FALSE, FALSE, 0);
  gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), GTK_RELIEF_NONE);

  temp = elysium_pixmap_file (g_strconcat (theme, "/back.png", NULL));
  window->back_tool_item =
    gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
			     NULL, _("Go Back"),
			     _("Back"),
			     gnome_stock_pixmap_widget (toolbar, temp),
			     encompass_history_go_back, window);
  if (temp) {
    g_free (temp);
  }
  gtk_widget_show (window->back_tool_item);
  gtk_widget_set_sensitive (window->back_tool_item, FALSE);

  gtk_widget_show (toolbar);
  gtk_widget_show (hbox);
  
  temp = elysium_pixmap_file (g_strconcat (theme, "/forward.png", NULL));
  window->fore_tool_item =
    gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
			     NULL, _("Go Forward"),
			     _("Forward"),
			     gnome_stock_pixmap_widget (toolbar, temp),
			     encompass_history_go_forward, window);
  if (temp) {
    g_free (temp);
  }
  gtk_widget_show (window->fore_tool_item);
  gtk_widget_set_sensitive (window->fore_tool_item, FALSE);

  gtk_widget_show (toolbar);
  gtk_widget_show (hbox);

  temp = elysium_pixmap_file (g_strconcat (theme, "/stop.png", NULL));
  window->stop_tool_item =
    gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
			     NULL, _("Stop loading"),
			     _("Stop"),
			     gnome_stock_pixmap_widget (toolbar, temp),
			     encompass_http_stop, window);
  if (temp) {
    g_free (temp);
  }
  gtk_widget_show (window->stop_tool_item);
  gtk_widget_set_sensitive (window->stop_tool_item, FALSE);

  gtk_widget_show (toolbar);
  gtk_widget_show (hbox);
  
  gnome_dock_add_item (GNOME_DOCK (GNOME_APP (window->window)->dock),
		       GNOME_DOCK_ITEM (window->navbar), GNOME_DOCK_TOP,
		       2, 0, 0, FALSE);
  gtk_widget_show (window->navbar);

  /* Create the URL Toolbar */
  window->urlbar = gnome_dock_item_new ("urlbar", GNOME_DOCK_ITEM_BEH_EXCLUSIVE);
  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (window->urlbar), hbox);
  gtk_container_border_width (GTK_CONTAINER (window->urlbar), 2);

  temp = elysium_pixmap_file ("encompass/url.png");
  urlpixmap = gnome_stock_pixmap_widget_at_size (window->urlbar, temp, 22, 22);
  gtk_box_pack_start (GTK_BOX (hbox), urlpixmap, FALSE, FALSE, 0);
  gtk_widget_show (urlpixmap);
  if (temp) {
    g_free (temp);
  }

  label = gtk_label_new (_("Location:"));
  gtk_box_pack_start (GTK_BOX (hbox), label,
		      FALSE, FALSE, 0);
  gtk_widget_show (label);

  window->entry = gtk_entry_new ();
  gtk_signal_connect (GTK_OBJECT (window->entry), "activate",
		      GTK_SIGNAL_FUNC (encompass_entry_goto_url), window);
  gtk_box_pack_start (GTK_BOX (hbox), window->entry, TRUE, TRUE, 0);
  gtk_widget_show (window->entry);
  gtk_widget_show (hbox);

  gnome_dock_add_item (GNOME_DOCK (GNOME_APP (window->window)->dock),
		       GNOME_DOCK_ITEM (window->urlbar), GNOME_DOCK_TOP,
		       2, 0, 0, FALSE);
  gtk_widget_show (window->urlbar);

  if (theme) {
    g_free (theme);
  }
}
