#include "encompass.h"

static GNOME_Encompass_Shell * shell = NULL;

void encompass_main_quit (GtkWidget * widget, BrowserWindow * window) {
  gtk_main_quit ();
}

static gint encompass_idle_cb (void * data) {
  GSList * uri_list;
  GNOME_Encompass_Shell corba_shell;
  CORBA_Environment ev;
  gboolean restored;

  CORBA_exception_init (&ev);

  uri_list = (GSList *) data;
  num_wins = 0;

  shell = encompass_shell_new ();
  if (shell == NULL) {
    corba_shell = oaf_activate_from_id ("OAFIID:GNOME_Encompass_Shell",
					0, NULL, &ev);
    if (ev._major != CORBA_NO_EXCEPTION || corba_shell == CORBA_OBJECT_NIL) {
      CORBA_exception_free (&ev);
      g_error ("Couldn't initialize the Encompass Shell");
      gtk_main_quit ();
      return FALSE;
    }
    restored = FALSE;
  } else {
    /*
    if (uri_list == NULL) {
      restored = TRUE;
    } else {
      restored = FALSE;
    }
    */
    corba_shell = bonobo_object_corba_objref (BONOBO_OBJECT (shell));
    corba_shell = CORBA_Object_duplicate (corba_shell, &ev);
    Bonobo_Unknown_ref (corba_shell, &ev);
  }

  if (uri_list == NULL) {
    GNOME_Encompass_Shell_handleURI (corba_shell, "", &ev);
  } else {
    GSList * p;

    for (p = uri_list; p != NULL; p = p->next) {
      gchar * uri;

      uri = encompass_validate_uri ((gchar *) p->data);
      GNOME_Encompass_Shell_handleURI (corba_shell, uri, &ev);
    }
    /* maybe add some things here like reload-prefs? */
    g_slist_free (uri_list);
  }

  CORBA_exception_free (&ev);

  if (shell == NULL) {
    gtk_main_quit ();
  }
  return FALSE;
}

int main (int argc, char ** argv) {
  CORBA_Environment ev;
  CORBA_ORB orb;
  GSList * uri_list;
  poptContext ctx;
  const gchar ** args;

  CORBA_exception_init (&ev);

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, GNOME_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

  gnome_init_with_popt_table (PACKAGE, VERSION, argc, argv,
			      oaf_popt_options, 0, &ctx);

  orb = oaf_init (argc, argv);

  if (bonobo_init (orb, NULL, NULL) == FALSE) {
    g_error ("Couldn't initialize Bonobo");
  }

  g_thread_init (NULL);

  gdk_rgb_init ();
  gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
  gtk_widget_set_default_visual (gdk_rgb_get_visual ());

  uri_list = NULL;

  args = poptGetArgs (ctx);
  if (args != NULL) {
    const gchar ** p;

    for (p = args; *p != NULL; p++) {
      uri_list = g_slist_prepend (uri_list, (char *) *p);
    }
  }

  gtk_idle_add (encompass_idle_cb, uri_list);
  bonobo_main ();

  return 0;
}
